/* $Id: lksteh.c,v 1.5 2002/06/28 11:27:53 indoh Exp $ */
/* command/lksteh.c */

/************************************************************
 *
 * COPYRIGHT (C) HITACHI,LTD. 2002 ALL RIGHTS RESERVED.
 * WRITTEN BY HITACHI SYSTEMS DEVELOPMENT LABORATORY,
 *            HITACHI CENTRAL RESEARCH LABORATORY.
 *
 * Created by K.Hatasaki <keisukeh@crl.hitachi.co.jp>
 *
 ************************************************************/

#include "command.h"

#define NAME	"lksteh"
#define VERSION "1.1"

#define NR_CMD_MAX	128

void usage(void);

void call_usage(int* , char**);
void call_version(int*, char**);
void call_list(int*, char**);
void call_ctrl(int*, char**);

int main(int argc, char **argv)
{
	int i;

	struct command cmd[NR_CMD_MAX] = 
	{
		[0] = {"help", &call_usage},
		[1] = {"ver", &call_version},
		[2] = {"version", &call_version},
		[3] = {"list", &call_list},
		[4] = {"ls", &call_list},
		[5] = {"control", &call_ctrl},
		[6] = {"ctrl", &call_ctrl},
		[7] = {"c", &call_ctrl},
		[8] = {"", NULL},
	};

	if (argc == 1) { /* no command */
		usage();
	} else {
		for (i = 0; i < NR_CMD_MAX; i++) {
			if (!cmd[i].func)
				break;
			if (!strcmp(cmd[i].name, argv[1])) {
				cmd[i].func(&argc, argv);
				break;
			}
		}
		if (!cmd[i].func)
			badcommand(argv[1]);
	}

	exit(0);
}

void usage(void)
{
	printf("Invoke an event-handler-control-function.\n");
	printf("Usage:\n\t%s command [option(s)]\n\n", NAME);
	printf("<COMMAND>\n");
	printf("control/ctrl/c\tInvoke an event-handler-control-function.\n");
	printf("\t\t<OPTION>\n");
	printf("\t\t-e event_handler_id [-f file_name]\n");
	printf("list/ls\t\tOutput a list of all event-handlers.\n");
	printf("version/ver\tPrint version information.\n");
	printf("help\t\tPrint this message.\n");
	exit(0);
}

void call_usage(int *argc, char **argv)
{
	usage();
}

void call_version(int *argc, char **argv)
{
	version(NAME, VERSION);
}


void call_list(int *argc, char **argv)
{
	evhandler_list();
}

void call_ctrl(int *argc, char **argv)
{
	int c;
	int id_set = 0;
	int name_set = 0;
	lkst_evhandler_id id;
	char file_name[255];

	while((c = getopt(*argc, argv, "e:f:")) != EOF) {
		switch(c) {
		case 'e' :
			id_set = 1;
			id = atoi(optarg);
			break;
		case 'f' :
			strncpy(file_name, optarg, 255);
			name_set = 1;
			break;
		default :
			usage();
			break;
		}
	}

	if (!id_set)
		needopt();

	evhandler_ctrl(id, file_name, name_set);
}
